from django.contrib.auth.hashers import check_password
from django.core.exceptions import PermissionDenied

import graphene
from graphene_django import DjangoObjectType

from aleksis.core.util.core_helpers import get_site_preferences


class SubmitSharedSecretMutation(graphene.Mutation):
    class Arguments:
        shared_secret = graphene.String(required=True)  # noqa

    correct = graphene.Boolean()

    @classmethod
    def mutate(cls, root, info, shared_secret):  # noqa
        if not info.context.session.get("maka_shared_secret_correct", False):
            info.context.session["maka_shared_secret_correct"] = check_password(
                shared_secret, get_site_preferences()["maka__shared_secret"]
            )
            return cls(correct=info.context.session.get("maka_shared_secret_correct", False))
        return cls(correct=True)


class SharedSecretObjectType(DjangoObjectType):
    class Meta:
        abstract = True

    @classmethod
    def get_queryset(cls, queryset, info):
        if not info.context.session.get("maka_shared_secret_correct", False):
            raise PermissionDenied()

        return super().get_queryset(queryset, info)


class SharedSecretBatchCreateMixin:
    """Mixin for maka shared secret checking during batch create mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        if info.context.session.get("maka_shared_secret_correct", False):
            return

        raise PermissionDenied()


class SharedSecretBatchPatchMixin:
    """Mixin for maka shared secret checking during batch patch mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        if info.context.session.get("maka_shared_secret_correct", False):
            return

        raise PermissionDenied()


class SharedSecretBatchDeleteMixin:
    """Mixin for maka shared secret checking during batch delete mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        if info.context.session.get("maka_shared_secret_correct", False):
            return

        raise PermissionDenied()
